==> ask-cli-1.7.6.tgz: package/lib/commands/create-hosted-skill/handler.js <==
const apiWrapper = require('@src/api/api-wrapper');
const clone = require('@src/clone/clone');
const CONSTANTS = require('@src/utils/constants');
const displayUtil = require('@src/utils/display-utility');
const HostedSkillController = require('@src/controller/hosted-skill-controller/index');
const profileHelper = require('@src/utils/profile-helper');
const ui = require('./ui');
const skillManifest = require('./hosted-skill-init-manifest');
const helper = require('./helper');

module.exports = {
    handleOptions,
};

function handleOptions(options) {
    validateOption(options, (validationError) => {
        if (validationError) {
            console.error(validationError);
            process.exit(1);
            return;
        }

        const profile = profileHelper.runtimeProfile(options.profile);
        const hostedSkillController = new HostedSkillController({ profile });
        helper.validateUserQualification(options, hostedSkillController, (validateQualificationError) => {
            if (validateQualificationError) {
                console.error(validateQualificationError);
                process.exit(1);
                return;
            }
            createSkill(options, hostedSkillController, (createSkillError, skillId) => {
                if (createSkillError) {
                    console.error(createSkillError);
                    if (skillId) {
                        apiWrapper.callDeleteSkill(skillId, profile, options.debug, () => {
                            process.exit(1);
                        });
                    } else {
                        process.exit(1);
                    }
                    return;
                }

                console.log('Hosted skill provisioning finished.');

                ui.getAutoClone(options.autoClone, (getAutoCloneError, autoClone) => {
                    if (getAutoCloneError) {
                        console.error(getAutoCloneError);
                        process.exit(1);
                        return;
                    }
                    if (autoClone) {
                        clone.singleSkillCloneProcess(skillId, profile, options.debug);
                    } else {
                        console.log(`Hosted skill created successfully. Skill-Id: ${skillId}`);
                        console.log(`Please run "ask clone --skill-id ${skillId}" to start your development.`);
                    }
                });
            });
        });
    });
}

/**
 * Create hosted skill and wait for provisioning success status.
 *
 * @param options
 * @param hostedSkillController
 * @param callback (error, skillId)
 * @return error and skillId if the skill has been created
 */
function createSkill(options, hostedSkillController, callback) {
    console.log('-------------------- Create Alexa Hosted Skill Project --------------------');
    ui.getSkillName(options.skillName, (getSkillNameError, userSkillName) => {
        if (getSkillNameError) {
            callback(getSkillNameError, null);
            return;
        }

        ui.getRunTime(options.runtime, (getRunTimeError, runTime) => {
            if (getRunTimeError) {
                callback(getRunTimeError, null);
                return;
            }

            const hostedSkillRuntime = CONSTANTS.HOSTED_SKILL.RUNTIME[runTime];

            const manifest = getManifest(userSkillName);
            const input = {
                manifest,
                runtime: hostedSkillRuntime,
            };

            const createSkillSpinner = displayUtil.Spinner('Creating your Alexa hosted skill. It will take about a minute.');
            createSkillSpinner.start();
            hostedSkillController.createSkill(input, options.debug, (error, skillId) => {
                if (error) {
                    createSkillSpinner.stop();
                    callback(error, skillId);
                    return;
                }

                hostedSkillController.pollingSkillStatus(skillId, options.debug, (pollingSkillStatusError, response) => {
                    createSkillSpinner.stop();
                    if (pollingSkillStatusError) {
                        callback(pollingSkillStatusError, skillId);
                        return;
                    }
                    handleSkillStatus(response, skillId, (handleSkillStatusError) => {
                        callback(handleSkillStatusError, skillId);
                    });
                });
            });
        });
    });
}

/**
 * Update hosted skill manifest template with customer input skill name
 * and return the manifest object.
 *
 * @param skillName
 * @return manifest
 */
function getManifest(skillName) {
    const manifest = skillManifest.createSkill;
    const { locales } = manifest.manifest.publishingInformation;

    Object.keys(locales).forEach((locale) => {
        manifest.manifest.publishingInformation.locales[locale].name = skillName;
    });
    return manifest;
}

/**
 * Process getSkillStatus result for new hosted skill manifest and provisioning status.
 *
 * @param response
 * @param skillId
 * @param callback (error, skillId)
 * @return callback with skillId or error
 */
function handleSkillStatus(response, skillId, callback) {
    if (!response) {
        callback('[Fatal]: Response from the creation of hosted skill is not valid.', skillId);
    } else if (response.provisioning.status === CONSTANTS.SKILL.PROVISIONING_STATUS.SUCCESS
        && response.interactionModel.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.SUCCESS
        && response.manifest.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.SUCCESS) {
        callback(null, skillId);
    } else if (response.provisioning.status === CONSTANTS.SKILL.PROVISIONING_STATUS.IN_PROGRESS
        || response.interactionModel.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.IN_PROGRESS
        || response.manifest.status === CONSTANTS.SKILL.MANIFEST_STATUS.IN_PROGRESS) {
        callback('[Error]: Timeout when checking the status of hosted-skill creation. Please try again.', skillId);
    } else {
        console.error('[Error]: Failed to create hosted skill for the following reason:');
        if (response.provisioning.status === CONSTANTS.SKILL.PROVISIONING_STATUS.FAILURE) {
            console.error('Skill provisioning step failed');
        }
        if (response.interactionModel.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.FAILURE) {
            console.error('Skill interaction model building step failed');
        }
        if (response.manifest.status === CONSTANTS.SKILL.MANIFEST_STATUS.FAILURE) {
            console.error('Skill manifest building step failed');
        }

        callback('[Error]: Infrastructure provision for the hosted skill failed. Please try again.', skillId);
    }
}

function validateOption(options, callback) {
    if (typeof options === 'string') {
        callback('[Error]: Invalid command. Please run "ask create-hosted-skill -h" for help.');
        return;
    }

    if (!(options.skillName && options.runtime && options.autoClone)
        && (options.skillName || options.runtime || options.autoClone)) {
        callback('[Error]: Invalid command usage. --skill-name, --runtime and --auto-clone must be either be set together or all absent.');
        return;
    }

    if (options.runtime && !Object.keys(CONSTANTS.HOSTED_SKILL.RUNTIME).includes(options.runtime)) {
        callback('[Error]: Invalid runtime option. Please run "ask create-hosted-skill -h" for help.');
        return;
    }

    if (options.autoClone && !(options.autoClone === 'true' || options.autoClone === 'false')) {
        callback('[Error]: Invalid auto clone option. Please run "ask create-hosted-skill -h" for help.');
        return;
    }
    callback();
}

==> ask-cli-1.7.12.tgz: package/lib/commands/create-hosted-skill/handler.js <==
const apiWrapper = require('@src/api/api-wrapper');
const clone = require('@src/clone/clone');
const CONSTANTS = require('@src/utils/constants');
const displayUtil = require('@src/utils/display-utility');
const HostedSkillController = require('@src/controller/hosted-skill-controller/index');
const profileHelper = require('@src/utils/profile-helper');
const ui = require('./ui');
const skillManifest = require('./hosted-skill-init-manifest');
const helper = require('./helper');

module.exports = {
    handleOptions,
};

function handleOptions(options) {
    validateOption(options, (validationError) => {
        if (validationError) {
            console.error(validationError);
            process.exit(1);
            return;
        }

        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        const hostedSkillController = new HostedSkillController({ profile });
        helper.validateUserQualification(options, vendorId, hostedSkillController, (validateQualificationError) => {
            if (validateQualificationError) {
                console.error(validateQualificationError);
                process.exit(1);
                return;
            }
            createSkill(options, hostedSkillController, (createSkillError, skillId) => {
                if (createSkillError) {
                    console.error(createSkillError);
                    if (skillId) {
                        apiWrapper.callDeleteSkill(skillId, profile, options.debug, () => {
                            process.exit(1);
                        });
                    } else {
                        process.exit(1);
                    }
                    return;
                }

                console.log('Hosted skill provisioning finished.');

                ui.getAutoClone(options.autoClone, (getAutoCloneError, autoClone) => {
                    if (getAutoCloneError) {
                        console.error(getAutoCloneError);
                        process.exit(1);
                        return;
                    }
                    if (autoClone) {
                        clone.singleSkillCloneProcess(skillId, profile, options.debug);
                    } else {
                        console.log(`Hosted skill created successfully. Skill-Id: ${skillId}`);
                        console.log(`Please run "ask clone --skill-id ${skillId}" to start your development.`);
                    }
                });
            });
        });
    });
}

/**
 * Create hosted skill and wait for provisioning success status.
 *
 * @param options
 * @param hostedSkillController
 * @param callback (error, skillId)
 * @return error and skillId if the skill has been created
 */
function createSkill(options, hostedSkillController, callback) {
    console.log('-------------------- Create Alexa Hosted Skill Project --------------------');
    ui.getSkillName(options.skillName, (getSkillNameError, userSkillName) => {
        if (getSkillNameError) {
            callback(getSkillNameError, null);
            return;
        }

        ui.getRunTime(options.runtime, (getRunTimeError, runTime) => {
            if (getRunTimeError) {
                callback(getRunTimeError, null);
                return;
            }

            const hostedSkillRuntime = CONSTANTS.HOSTED_SKILL.RUNTIME[runTime];

            const manifest = getManifest(userSkillName);
            const input = {
                manifest,
                runtime: hostedSkillRuntime,
            };

            const createSkillSpinner = displayUtil.Spinner('Creating your Alexa hosted skill. It will take about a minute.');
            createSkillSpinner.start();
            hostedSkillController.createSkill(input, options.debug, (error, skillId) => {
                if (error) {
                    createSkillSpinner.stop();
                    callback(error, skillId);
                    return;
                }

                hostedSkillController.pollingSkillStatus(skillId, options.debug, (pollingSkillStatusError, response) => {
                    createSkillSpinner.stop();
                    if (pollingSkillStatusError) {
                        callback(pollingSkillStatusError, skillId);
                        return;
                    }
                    handleSkillStatus(response, skillId, (handleSkillStatusError) => {
                        callback(handleSkillStatusError, skillId);
                    });
                });
            });
        });
    });
}

/**
 * Update hosted skill manifest template with customer input skill name
 * and return the manifest object.
 *
 * @param skillName
 * @return manifest
 */
function getManifest(skillName) {
    const manifest = skillManifest.createSkill;
    const { locales } = manifest.manifest.publishingInformation;

    Object.keys(locales).forEach((locale) => {
        manifest.manifest.publishingInformation.locales[locale].name = skillName;
    });
    return manifest;
}

/**
 * Process getSkillStatus result for new hosted skill manifest and provisioning status.
 *
 * @param response
 * @param skillId
 * @param callback (error, skillId)
 * @return callback with skillId or error
 */
function handleSkillStatus(response, skillId, callback) {
    if (!response) {
        callback('[Fatal]: Response from the creation of hosted skill is not valid.', skillId);
    } else if (response.provisioning.status === CONSTANTS.SKILL.PROVISIONING_STATUS.SUCCESS
        && response.interactionModel.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.SUCCESS
        && response.manifest.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.SUCCESS) {
        callback(null, skillId);
    } else if (response.provisioning.status === CONSTANTS.SKILL.PROVISIONING_STATUS.IN_PROGRESS
        || response.interactionModel.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.IN_PROGRESS
        || response.manifest.status === CONSTANTS.SKILL.MANIFEST_STATUS.IN_PROGRESS) {
        callback('[Error]: Timeout when checking the status of hosted-skill creation. Please try again.', skillId);
    } else {
        console.error('[Error]: Failed to create hosted skill for the following reason:');
        if (response.provisioning.status === CONSTANTS.SKILL.PROVISIONING_STATUS.FAILURE) {
            console.error('Skill provisioning step failed');
        }
        if (response.interactionModel.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.FAILURE) {
            console.error('Skill interaction model building step failed');
        }
        if (response.manifest.status === CONSTANTS.SKILL.MANIFEST_STATUS.FAILURE) {
            console.error('Skill manifest building step failed');
        }

        callback('[Error]: Infrastructure provision for the hosted skill failed. Please try again.', skillId);
    }
}

function validateOption(options, callback) {
    if (typeof options === 'string') {
        callback('[Error]: Invalid command. Please run "ask create-hosted-skill -h" for help.');
        return;
    }

    if (!(options.skillName && options.runtime && options.autoClone)
        && (options.skillName || options.runtime || options.autoClone)) {
        callback('[Error]: Invalid command usage. --skill-name, --runtime and --auto-clone must be either be set together or all absent.');
        return;
    }

    if (options.runtime && !Object.keys(CONSTANTS.HOSTED_SKILL.RUNTIME).includes(options.runtime)) {
        callback('[Error]: Invalid runtime option. Please run "ask create-hosted-skill -h" for help.');
        return;
    }

    if (options.autoClone && !(options.autoClone === 'true' || options.autoClone === 'false')) {
        callback('[Error]: Invalid auto clone option. Please run "ask create-hosted-skill -h" for help.');
        return;
    }
    callback();
}
